@@ -1,22 +1,37 @@
 #!/usr/dist/bin/wishx -f
 
+######################################################################
 #
-# main
+# dontspaceRules.tcl
 #
-set rcsid(dontspaceRules.tcl) {$Id: dontspaceRules.tcl,v 2.9 1994/06/16 05:55:03 johnh Exp $}
+# Copyright (C) 1993,1994 by John Heidemann <johnh@ficus.cs.ucla.edu>
+# All rights reserved.  See the main dontspace file for a full copyright
+# notice.
+#
+# $Id: dontspaceRules.tcl,v 2.16 1997/03/26 17:46:26 johnh Exp $
+#
+######################################################################
+
+set rcsid(dontspaceRules.tcl) {$Id: dontspaceRules.tcl,v 2.16 1997/03/26 17:46:26 johnh Exp $}
 
 
 
 proc main {} {
-	global game table
+	global game table tk_version argv
 
 	source "$game(sourcedir)/table.tcl"
 	source "$game(sourcedir)/score.tcl"
 	source "$game(sourcedir)/help.tcl"
 	source "$game(sourcedir)/dontspaceMenus.tcl"
 
-	bind . <Any-Enter> {focus [focus default]}
+	if {$tk_version < 4.0} {
+		bind . <Any-Enter> {focus [focus default]}
+	}
 	mkTableDefaults
+	if {[winfo screenheight .] < 600} {
+		set table(gutter) 10
+		set table(stackedCardOffset) 20
+	}
 
 	set game(numTableColumns) 8
 	set game(zeroBasedList) {0 1 2 3 4 5 6 7 8 9 10 11 12}
@@ -34,7 +49,11 @@ proc main {} {
 	if { $cardWidth < 9 } { set cardWidth 9 }
 	# Largest table is
 	set tableWidth  [expr $cardWidth*$table(cardWidth)+$cardWidth*$table(gutter) ]
-	set tableHeight [expr 2*$table(cardHeight)+4*$table(gutter)+15*$table(stackedCardOffset) ]
+	# Longest possible chain of cards: 7 card start w/a king on the bottom
+	# + 13 cards down from the king.  This configuration is extremely
+	# unlikely and makes the table *way* too big for normal play.
+	# As a compromise we allow 15 stacked cards.
+	set tableHeight [expr 2*$table(cardHeight)+3.5*$table(gutter)+14*$table(stackedCardOffset) ]
 
 	set game(tableW) [mkTable $tableWidth $tableHeight gamePreActionProc]
 	set game(menuW) [mkMenus]
@@ -53,7 +72,13 @@ proc main {} {
 		$table(gutter) game(scoreMethod)
 
 	# a game
-	mkNewGame
+	if {[llength $argv] <= 0} {
+		mkNewGame
+	} else {
+		set seed [lindex $argv 0]
+		# puts "Old game key $seed."
+		mkOldGame $seed
+	}
 }
 
 proc mkUnchangingItems {} {
@@ -85,7 +110,7 @@ proc mkUnchangingItems {} {
 	foreach i $game(tableList) {
 		set game(tableId,$i) [createItemBitmap \
 			[expr $table(gutter)+($i+$offset)*($table(cardWidth)+$table(gutter))] \
-			[expr 3*$table(gutter)+$table(cardHeight)] "space"]
+			[expr 2.5*$table(gutter)+$table(cardHeight)] "space"]
 	}
 
 	# Make the notice text.
@@ -351,6 +376,7 @@ proc enableVsDoubleClicking {w vs} {
 }
 
 proc singleOnlyAcceptProc {w target src srcFriends} {
+	# debugLog "singleOnlyAcceptProc $w $target $src $srcFriends"
 	if { [llength $srcFriends] != 0 } {
 		return 0
 	} else {
@@ -359,6 +385,7 @@ proc singleOnlyAcceptProc {w target src srcFriends} {
 }
 
 proc tableSpaceAcceptProc {w target src srcFriends} {
+	# debugLog "tableSpaceAcceptProc $w $target $src $srcFriends"
 	if {[dangerousMove $w $target $src $srcFriends]} {
 		return 0
 	} else {
@@ -367,6 +394,7 @@ proc tableSpaceAcceptProc {w target src srcFriends} {
 }
 
 proc spaceAdoptChild {w target src srcFriends closure} {
+	# debugLog "spaceAdoptChild $w $target $src $srcFriends"
 	global game
 
 	if { [llength $srcFriends] != 0} {
@@ -396,6 +424,7 @@ proc spaceAdoptChild {w target src srcFriends closure} {
 }
 
 proc spaceOrphanChild {w target src srcFriends closure} {
+	# debugLog "spaceOrphanChild $w $target $src $srcFriends"
 	global game
 
 	# Enable the space.
@@ -410,6 +439,7 @@ proc spaceOrphanChild {w target src srcFriends closure} {
 proc foundationAdoptChild {w target src srcFriends closure} {
 	global game
 
+	# debugLog "foundationAdoptChild $w $target $src $srcFriends $closure"
 	if { [llength $srcFriends] != 0} {
 		error "foundationAdoptChild: child with friends"
 	}
@@ -418,6 +448,7 @@ proc foundationAdoptChild {w target src srcFriends closure} {
 		-dragTargetAcceptGlob {} \
 		-child $src
 	# Move the child.
+	# debugLogPause "foundationAdoptChild: moveCardOnCard $w $src $target"
 	moveCardOnCard $w $src $target
 
 	# Make the child accept the next card and otherwise unusable.
@@ -426,6 +457,7 @@ proc foundationAdoptChild {w target src srcFriends closure} {
 	set s [string index $vs 1]
 	set newV [figureNextValue $v 1]
 	set column [getCard $w $target -sublocation]
+	# debugLogPause "foundationAdoptChild: setCard (colmn: $column)"
 	setCard $w $src \
 		-location foundation \
 		-sublocation $column \
@@ -434,6 +466,7 @@ proc foundationAdoptChild {w target src srcFriends closure} {
 		-childOffsetProc samePlaceChildOffsetProc \
 		-dtag dragableCard \
 		-dtag doubleClickableCard
+	unhilightCard $src $w {} {}
 	# Indicate the new top.
 	set game(foundationTopId,$column) $src
 
@@ -442,6 +475,7 @@ proc foundationAdoptChild {w target src srcFriends closure} {
 
 	# Change dragability, possibly making more cards double clickable.
 	fixTableDragability $w
+	# debugLogPause "foundationAdoptChild: enablePatternDoubleClicking"
 	enablePatternDoubleClicking $w "$newV$s"
 
 	# Finally, check for win.
@@ -468,6 +502,7 @@ proc foundationAccepts {w id} {
 proc tableAdoptChild {w target src srcFriends closure} {
 	global game
 
+	# debugLog "tableAdoptChild $w $target $src $srcFriends"
 	if { [llength $srcFriends] != 0} {
 		moveMultipleCards $w $target $src $srcFriends $closure
 		return
@@ -507,6 +542,7 @@ proc tableAdoptChild {w target src srcFriends closure} {
 proc tableSpaceOrphanChild {w target src srcFriends closure} {
 	global game
 
+	# debugLog "tableSpaceOrphanChild $w $target $src $srcFriends $closure"
 	# Enable the table.
 	setCard $w $target \
 		-dragTargetAcceptGlob {??}
@@ -530,6 +566,7 @@ proc tableCardOrphanChild {w target src srcFriends closure} {
 }
 
 proc tableOrphanChild {w target src srcFriends closure} {
+	# debugLog "tableOrphanChild $w $target $src $srcFriends $closure"
 	# The target no longer has a friend.
 	setCard $w $target \
 		-dragFindFriendsClosure {} \
@@ -553,7 +590,7 @@ proc tableOrphanChild {w target src srcFriends closure} {
 }
 
 proc cardDoubleClick {itemId w x y closure} {
-#	puts "cardDoubleClick: $w $itemId"
+	# debugLog "cardDoubleClick $itemId $w $x $y $closure"
 	set target [foundationAccepts $w $itemId]
 	if { $target == {} } {
 		error "cardDoubleClick: double-click on card with no viable destination."
